@@ -1,5 +1,5 @@
 #!/bin/env python
-# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development Company LP
+# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP
 """Monitoring Agent plugin for HTTP/API checks.
 
 """
@@ -22,6 +22,12 @@ import monasca_agent.common.keystone as keystone
 class HTTPCheck(services_checks.ServicesCheck):
 
     def __init__(self, name, init_config, agent_config, instances=None):
+        self._socket_errors = set()
+        self._response_not_ready = set()
+        self._general_exception = set()
+        self._invalid_token = set()
+        self._warn_msg = set()
+
         super(HTTPCheck, self).__init__(name, init_config, agent_config,
                                         instances)
         # init the keystone client if instance has use_keystone
@@ -62,11 +68,6 @@ class HTTPCheck(services_checks.ServicesCheck):
         return url, username, password, timeout, headers, response_time, \
             ssl, use_keystone, keystone_config, instance_name
 
-    def _create_status_event(self, status, msg, instance):
-        """Does nothing: status events are not yet supported by Mon API.
-        """
-        return
-
     def _http_check(self, instance):
         addr, username, password, timeout, headers, response_time, \
             disable_ssl_validation, use_keystone, keystone_config, \
@@ -85,10 +86,10 @@ class HTTPCheck(services_checks.ServicesCheck):
                     headers["X-Auth-Token"] = token
                     headers["Content-type"] = "application/json"
                 else:
-                    warning_string = """Unable to get token. Keystone API
-                    server may be down. Skipping check for {0}""".format(addr)
-                    self.log.warning(warning_string)
-                    return False, warning_string
+                    error_msg = "Unable to get token. Keystone API server may be down."
+                    warn_string = '{0} Skipping check for {1}'.format(error_msg, addr)
+                    self.log.warning(warn_string)
+                    return False, error_msg
             try:
                 self.log.debug("Connecting to %s" % addr)
                 if disable_ssl_validation:
@@ -100,24 +101,32 @@ class HTTPCheck(services_checks.ServicesCheck):
 
             except (socket.timeout, HttpLib2Error, socket.error) as e:
                 length = int((time.time() - start) * 1000)
-                warn_string = '{0} is DOWN, error: {1}. Connection failed ' \
-                              'after {2} ms'.format(addr, repr(e), length)
-                self.log.warn(warn_string)
-                return False, warn_string
+                error_msg = 'error: {0}. Connection failed after {1} ' \
+                            'ms'.format(repr(e), length)
+                if addr not in self._socket_errors:
+                    self._socket_errors.add(addr)
+                    warn_string = '{0} is DOWN, {1}'.format(addr, error_msg)
+                    self.log.warn(warn_string)
+                return False, error_msg
 
             except httplib.ResponseNotReady as e:
                 length = int((time.time() - start) * 1000)
-                warn_string = '{0} is DOWN, error: {1}. Network is not ' \
-                              'routable after {2} ms'.format(addr, repr(e),
-                                                             length)
-                self.log.warn(warn_string)
-                return False, warn_string
+                error_msg = 'error: {0}. Network is not routable after {1} ' \
+                            'ms'.format(repr(e), length)
+                if addr not in self._response_not_ready:
+                    self._response_not_ready.add(addr)
+                    warn_string = '{0} is DOWN, {1}'.format(addr, error_msg)
+                    self.log.warn(warn_string)
+                return False, error_msg
 
             except Exception as e:
                 length = int((time.time() - start) * 1000)
-                error_string = '{0} is DOWN, error: {1}. Connection failed after {2} ms'.format(addr, repr(e), length)
-                self.log.error(error_string)
-                return False, error_string
+                error_msg = 'error: {0}. Connection failed after {1} ms'.format(repr(e), length)
+                if addr not in self._general_exception:
+                    self._general_exception.add(addr)
+                    error_string = '{0} is DOWN, {1}'.format(addr, error_msg)
+                    self.log.error(error_string)
+                return False, error_msg
 
             if response_time:
                 # Stop the timer as early as possible
@@ -127,9 +136,12 @@ class HTTPCheck(services_checks.ServicesCheck):
             if int(resp.status) >= 400:
                 if use_keystone and int(resp.status) == 401:
                     if retry:
-                        error_string = '{0} is DOWN, unable to get a valid token to connect with'.format(addr)
-                        self.log.error(error_string)
-                        return False, error_string
+                        error_msg = 'unable to get a valid token to connect with'
+                        if addr not in self._invalid_token:
+                            self._invalid_token.add(addr)
+                            error_string = '{0} is DOWN, {1}'.format(addr, error_msg)
+                            self.log.error(error_string)
+                        return False, error_msg
                     else:
                         # Get a new token and retry
                         self.log.info("Token expired, getting new token and retrying...")
@@ -137,10 +149,19 @@ class HTTPCheck(services_checks.ServicesCheck):
                         ksclient.refresh_token()
                         continue
                 else:
-                    warn_string = '{0} is DOWN, error code: {1}'.\
-                        format(addr, str(resp.status))
-                    self.log.warn(warn_string)
-                    return False, warn_string
+                    warn_msg = 'error code: {0}'.format(str(resp.status))
+                    if addr not in self._warn_msg:
+                        self._warn_msg.add(addr)
+                        warn_string = '{0} is DOWN, {1}'.format(addr, warn_msg)
+                        self.log.warn(warn_string)
+                    return False, warn_msg
+
+            self._socket_errors.discard(addr)
+            self._invalid_token.discard(addr)
+            self._response_not_ready.discard(addr)
+            self._general_exception.discard(addr)
+            self._warn_msg.discard(addr)
+
             done = True
             return True, content
 
@@ -152,9 +173,13 @@ class HTTPCheck(services_checks.ServicesCheck):
 
         success, result_string = self._http_check(instance)
         if not success:
+            # maximum length of value_meta including {'error':''} is 2048
+            # Cutting it down to 1024 here so we don't clutter the
+            # database too much.
             self.gauge('http_status',
                        1,
-                       dimensions=dimensions)
+                       dimensions=dimensions,
+                       value_meta={'error': result_string[:1024]})
             return services_checks.Status.DOWN, result_string
 
         if pattern is not None:
@@ -163,10 +188,13 @@ class HTTPCheck(services_checks.ServicesCheck):
             else:
                 error_string = 'Pattern match failed! "{0}" not in "{1}"'.format(pattern, result_string)
                 self.log.info(error_string)
+                # maximum length of value_meta including {'error':''} is 2048
+                # Cutting it down to 1024 here so we don't clutter the
+                # database too much.
                 self.gauge('http_status',
                            1,
                            dimensions=dimensions,
-                           value_meta={'error': error_string[:2048]})
+                           value_meta={'error': error_string[:1024]})
                 return services_checks.Status.DOWN, error_string
 
         success_string = '{0} is UP'.format(addr)
