@@ -1,4 +1,4 @@
-# (C) Copyright 2016 Hewlett Packard Enterprise Development LP
+# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development LP
 """VCenter Only.
 
 Generic VCenter check. This check allows you to specify particular metrics that
@@ -62,6 +62,14 @@ class VCenterCheck(AgentCheck):
         self.is_new_session = True
         self._resource_moid_dict = {}
 
+    def stop(self):
+        """To be executed when the agent is being stopped to clean resources.
+        """
+        if self.session is not None:
+            self.session.logout()
+            self.session = None
+            self.is_new_session = True
+
     def _propset_dict(self, propset):
         """Turns a propset list into a dictionary
 
@@ -300,6 +308,7 @@ class VCenterCheck(AgentCheck):
     def check(self, instance):
         try:
             if self.is_new_session:
+                self.instance = instance
                 self.vcenter_ip = instance.get('vcenter_ip', None)
                 self.user = instance.get('username', None)
                 self.password = instance.get('password', None)
@@ -451,10 +460,13 @@ class VCenterCheck(AgentCheck):
 
     def _get_dims(self, cluster, mor_id):
         cluster_id = mor_id + "." + self.vc_uuid
-        return {
+        local_dimensions = {
             "vcenter_ip": self.vcenter_ip,
             "esx_cluster_id": cluster_id
         }
+        final_dimensions = self._set_dimensions(local_dimensions,
+                                                self.instance)
+        return final_dimensions
 
     @staticmethod
     def parse_agent_config(agentConfig):
